{*******************************************************}
{                                                       }
{                 Google Map \NX                 }
{                                                       }
{*******************************************************}
unit uGoogleMapBrowser;

interface

uses
  Windows, Classes, Forms, Controls, SysUtils, OleCtrls, SHDocVw;

type
  // Y[͈
  TGM_ZoomRange = 0..22;

  // @ ( / ܓxEox)
  TGM_SerchType = (gmstString, gmsLocation);

  // n}̎ (n} / ʐ^ / n} + ʐ^ / Google Earth / n`)
  TGM_MapType = (gmmtMap, gmmtSatellite, gmmtHybrid, gmmtGoogleEarth, gmmtTerrain);

  { TGoogleMapBrowser }
  TGoogleMapBrowser = class(TWebBrowser)
  private
    FDescription: string;
    FHostLanguage: string;
    FLatitude: Double;
    FLongitude: Double;
    FMapType: TGM_MapType;
    FParam: string;
    FSearchString: String;
    FSearchType: TGM_SerchType;
    FURL: string;
    FZoomLevel: TGM_ZoomRange;
    procedure BuildParams;
    function GetURL: string;
    function Getparent: TWinControl;
    procedure InitURL;
    procedure SetParent(const Value: TWinControl); reintroduce;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure ShowBlank;                                                                      // 󔒃y[W\
    procedure ShowMap;                                                                        // Google Map \
    procedure PageSetup;                                                                      // y[Wݒ_CAO\
    procedure Print(ShowDialog: Boolean = True);                                              //  (ShowDialog = False ő)
    property Parent: TWinControl read Getparent write SetParent;                              // Parent Ē`ATOleControl(WebBrowser).Parent ̂悤ȃLXgsvɂB
    property URL: string read GetURL;                                                         // \zꂽ URL (ReadOnly)
  published
    property Description: string read FDescription write FDescription;                        // EBhEɕ\
    property HostLanguage: string read FHostLanguage write FHostLanguage;                     // zXǧw (Ex.ja)
    property Latitude: Double read FLatitude write FLatitude;                                 // ܓx (SearchType = gmsLocation )
    property Longitude: Double read FLongitude write FLongitude;                              // ox (SearchType = gmsLocation )
    property MapType: TGM_MapType read FMapType write FMapType default gmmtMap;               // \n}̎
    property SearchString: string read FSearchString write FSearchString;                     //  (SearchType = gmstString )
    property SearchType: TGM_SerchType read FSearchType write FSearchType default gmstString; // @
    property ZoomLevel: TGM_ZoomRange read FZoomLevel write FZoomLevel default 14;            // Y[x
  end;

procedure Register;

implementation

{ TGoogleMapBrowser }

const
  GM_URL   = 'http://maps.google.co.jp/maps';
  GM_BLANK = 'about:blank';

// -----------------------------------------------------------------------------
// Constructor / Destructor
// -----------------------------------------------------------------------------

// RXgN^
constructor TGoogleMapBrowser.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  InitURL;
  FMapType    := gmmtMap;
  FSearchType := gmstString;
  FZoomLevel  := 14;
end;

// fXgN^
destructor TGoogleMapBrowser.Destroy;
begin
  inherited Destroy;
end;

// -----------------------------------------------------------------------------
// Methods (Internal)
// -----------------------------------------------------------------------------

// URL 
procedure TGoogleMapBrowser.InitURL;
begin
  FURL   := GM_BLANK;
  FParam := '';
end;

// p[^̍\z
procedure TGoogleMapBrowser.BuildParams;
// Google Map Parameters
// http://web.archive.org/web/20080718141342/http://mapki.com/wiki/Google_Map_Parameters
var
  ParamList: TStringList;
  HasDescription: Boolean;
  Desc: string;
  ParamType: string;
  i: Integer;
  function EscapeParentheses(s: string): string;
  const
    rsv_char: array [0..17] of char =
      (':', '/', '?', '#', '[', ']', '@', '!', '$', '&', '''', '(', ')', '*', '+', ',', ';', '=');
  var
    i: Integer;
    function toZenkaku(S: String):String;
    var
      Buf: array [0..255] of Char;
    begin
      Windows.LCMapString(GetUserDefaultLCID(), LCMAP_FULLWIDTH, PChar(S),
                          Length(S) + 1, Buf, Sizeof(Buf));
      result := Buf;
    end;
  begin
    result := Trim(s);
    for i:=Low(rsv_char) to High(rsv_char) do
      result := StringReplace(result, rsv_char[i], toZenkaku(rsv_char[i]), [rfReplaceAll]);
  end;
begin
  InitURL;
  ParamList := TstringList.Create;
  try
    // Query
    FDescription := Trim(FDescription);
    HasDescription := (FDescription <> '');
    case FSearchType of
      gmsLocation:
        ParamType := Format('loc:%f,%f', [FLatitude, FLongitude]);
    else
      ParamType := Trim(EscapeParentheses(SearchString));
    end;

    if HasDescription then
      Desc := Format('+(%s)', [EscapeParentheses(FDescription)])
    else
      Desc := '';
    ParamList.Values['q'] := ParamType + Desc;
    if not HasDescription then
      ParamList.Values['iwloc'] := 'J';

    // MapType
    case FMapType of
      gmmtSatellite:
        ParamType := 'k';
      gmmtHybrid:
        ParamType := 'h';
      gmmtGoogleEarth:
        ParamType := 'e';
      gmmtTerrain:
        ParamType := 'p';
    else
      ParamType := 'm';
    end;
    ParamList.Values['t'] := ParamType;

    // HostLanguage
    if Trim(FHostLanguage) <> '' then
      ParamList.Values['hl'] := Trim(FHostLanguage);

    // ZoomLevel
    ParamList.Values['z'] := IntToStr(FZoomLevel);

    // Output Mode
    ParamList.Values['output'] := 'embed';

    // FParam
    FParam := '';
    for i:=0 to ParamList.Count-1 do
      begin
        FParam := FParam + ParamList[i];
        if i < (ParamList.Count-1) then
          FParam := FParam + '&';
      end;

    // FURL
    FURL := GM_URL + '?' + FParam;
  finally
    ParamList.Free;
  end;
end;

// -----------------------------------------------------------------------------
// Access Methods (Property)
// -----------------------------------------------------------------------------

// ē Parent vpeB̃ANZX\bh (Getter)
function TGoogleMapBrowser.Getparent: TWinControl;
begin
  result := TOleControl(Self).Parent;
end;

// ē Parent vpeB̃ANZX\bh (Setter)
procedure TGoogleMapBrowser.SetParent(const Value: TWinControl);
begin
  TOleControl(Self).Parent := Value;
end;

// URL vpeB̃ANZX\bh (Getter) - ReadOnly
function TGoogleMapBrowser.GetURL: string;
begin
  BuildParams;
  result := FURL;
end;

// -----------------------------------------------------------------------------
// Methods
// -----------------------------------------------------------------------------

// 󔒃y[W\
procedure TGoogleMapBrowser.ShowBlank;
begin
  Self.Navigate(GM_BLANK);
end;

// Google Map ̕\
procedure TGoogleMapBrowser.ShowMap;
begin
  BuildParams;
  Self.Navigate(FURL);
end;

// y[Wݒ_CAOJ
procedure TGoogleMapBrowser.PageSetup;
var
  vIn, vOut: OleVariant;
begin
  // 󔒃y[WȂ甲
  if FURL = GM_BLANK then
    Exit;

  // y[Wݒ_CAOJ
  Self.ControlInterface.ExecWB(OLECMDID_PAGESETUP, OLECMDEXECOPT_PROMPTUSER, vIn, vOut);
end;

//  (ShowDialog = False ő)
procedure TGoogleMapBrowser.Print(ShowDialog: Boolean);
var
  vDialog, vIn, vOut: OleVariant;
begin
  // \ȏԂɂȂ܂ő҂
  while Self.Busy do
    Forms.Application.ProcessMessages;

  // 󔒃y[WȂ甲
  if URL = GM_BLANK then
    Exit;

  // 
  if ShowDialog then
    vDialog := OLECMDEXECOPT_PROMPTUSER
  else
    vDialog := OLECMDEXECOPT_DONTPROMPTUSER;
  Self.ControlInterface.ExecWB(OLECMDID_PRINT, vDialog, vIn, vOut);
end;

procedure Register;
begin
  RegisterComponents('Internet', [TGoogleMapBrowser]);
end;
end.
